@@ -1,6 +1,6 @@
 /*
  * ****************************************************************************
- * Copyright (c) 2013-2021, PyInstaller Development Team.
+ * Copyright (c) 2013-2022, PyInstaller Development Team.
  *
  * Distributed under the terms of the GNU General Public License (version 2
  * or later) with exception for distributing the bootloader.
@@ -248,6 +248,68 @@ pyi_pylib_set_runtime_opts(ARCHIVE_STATUS *status)
     return 0;
 }
 
+/* Enable UTF-8 mode as per PEP540.
+ * It seems Py_UTF8Mode must be set before Py_SetPath is called, but
+ * in practice, it is probably a good idea to call it before any
+ * Py_* functions are used
+ */
+static void
+pyi_pylib_set_pep540_utf8_mode()
+{
+    int enable_utf8_mode = -1;
+    char *env_utf8 = NULL;
+
+    /* Honor the setting via PYTHONUTF8 environment variable (valid
+     * values are 0 and 1, same as with python interpreter) */
+    env_utf8 = pyi_getenv("PYTHONUTF8");
+    if (env_utf8) {
+        if (strcmp(env_utf8, "0") == 0) {
+            enable_utf8_mode = 0;
+        } else if (strcmp(env_utf8, "1") == 0) {
+            enable_utf8_mode = 1;
+        } else {
+            OTHERERROR("Invalid value for PYTHONUTF8=%s; disabling utf-8 mode!\n", env_utf8);
+            enable_utf8_mode = 0;
+        }
+    }
+
+#ifndef _WIN32
+    /* On non-Windows, the C locale and the POSIX locale enable the
+     * UTF-8 Mode (PEP 540) */
+    if (enable_utf8_mode < 0) {
+        const char *lc_ctype = NULL;
+        char *orig_lc_ctype = NULL;
+
+        /* Get original value of LC_CTYPE. */
+        lc_ctype = setlocale(LC_CTYPE, NULL);
+        if (lc_ctype) {
+            orig_lc_ctype = strdup(lc_ctype);
+        }
+
+        /* Set user-preferred locale, and retrieve corresponding LC_CTYPE. */
+        lc_ctype = setlocale(LC_CTYPE, "");
+        if (lc_ctype != NULL && (strcmp(lc_ctype, "C") == 0 || strcmp(lc_ctype, "POSIX") == 0)) {
+            enable_utf8_mode = 1;
+        }
+
+        /* Restore old value. */
+        if (orig_lc_ctype) {
+            setlocale(LC_CTYPE, orig_lc_ctype);
+            free(orig_lc_ctype);
+        }
+    }
+#endif
+
+    /* Enable/disable UTF-8 mode */
+    if (enable_utf8_mode > 0) {
+        VS("LOADER: Enabling UTF-8 mode\n");
+        *PI_Py_UTF8Mode = 1;
+    } else {
+        *PI_Py_UTF8Mode = 0;
+    }
+}
+
+
 void
 pyi_free_wargv(wchar_t ** wargv)
 {
@@ -401,6 +463,11 @@ pyi_pylib_start_python(ARCHIVE_STATUS *status)
     static wchar_t pyhome_w[PATH_MAX + 1];
     static wchar_t progname_w[PATH_MAX + 1];
 
+    /* Enable PEP540 UTF-8 mode, if necessary. Must be done before Py_SetPath()
+     * is called for the setting to take effect (but we should probably also
+     * set it before pyi_locale_char2wchar() calls). */
+    pyi_pylib_set_pep540_utf8_mode();
+
     /* Decode using current locale */
     if (!pyi_locale_char2wchar(progname_w, status->executablename, PATH_MAX)) {
         FATALERROR("Failed to convert progname to wchar_t\n");
@@ -552,12 +619,7 @@ pyi_pylib_import_modules(ARCHIVE_STATUS *status)
 
             /* Unmarshall code object for module; we need to skip
                the pyc header */
-            if (pyvers >= 307) {
-                /* Python 3.7 changed header size to 16 bytes */
-                co = PI_PyMarshal_ReadObjectFromString((const char *) modbuf + 16, ptoc->ulen - 16);
-            } else {
-                co = PI_PyMarshal_ReadObjectFromString((const char *) modbuf + 12, ptoc->ulen - 12);
-            }
+            co = PI_PyMarshal_ReadObjectFromString((const char *) modbuf + 16, ptoc->ulen - 16);
 
             if (co != NULL) {
                 VS("LOADER: running unmarshalled code object for %s...\n", ptoc->name);
@@ -602,7 +664,7 @@ int
 pyi_pylib_install_zlib(ARCHIVE_STATUS *status, TOC *ptoc)
 {
     int rc = 0;
-    uint64_t zlibpos = status->pkgstart + ptoc->pos;
+    unsigned long long zlibpos = status->pkgstart + ptoc->pos;
     PyObject * sys_path, *zlib_entry, *archivename_obj;
 
     /* Note that sys.path contains PyUnicode on py3. Ensure
@@ -620,7 +682,7 @@ pyi_pylib_install_zlib(ARCHIVE_STATUS *status, TOC *ptoc)
      */
     archivename_obj = PI_PyUnicode_DecodeFSDefault(status->archivename);
 #endif
-    zlib_entry = PI_PyUnicode_FromFormat("%U?%" PRIu64, archivename_obj, zlibpos);
+    zlib_entry = PI_PyUnicode_FromFormat("%U?%llu", archivename_obj, zlibpos);
     PI_Py_DecRef(archivename_obj);
 
     sys_path = PI_PySys_GetObject("path");
