@@ -1,6 +1,6 @@
 /*
  * ****************************************************************************
- * Copyright (c) 2013-2021, PyInstaller Development Team.
+ * Copyright (c) 2013-2022, PyInstaller Development Team.
  *
  * Distributed under the terms of the GNU General Public License (version 2
  * or later) with exception for distributing the bootloader.
@@ -21,6 +21,10 @@
 
 #include "pyi_archive.h"
 
+#ifndef _WIN32
+#include <sys/types.h> /* pid_t */
+#endif
+
 // some platforms do not provide strnlen
 #ifndef HAVE_STRNLEN
 size_t strnlen(const char *str, size_t n);
@@ -51,6 +55,9 @@ dylib_t pyi_utils_dlopen(const char *dllpath);
 int pyi_utils_dlclose(dylib_t dll);
 int pyi_utils_create_child(const char *thisfile, const ARCHIVE_STATUS *status,
                            const int argc, char *const argv[]);
+#ifndef _WIN32
+pid_t pyi_utils_get_child_pid();
+#endif
 int pyi_utils_set_environment(const ARCHIVE_STATUS *status);
 
 #if !defined(_WIN32) && !defined(__APPLE__)
@@ -59,21 +66,10 @@ int pyi_utils_replace_process(const char *thisfile, const int argc, char *const
 
 /* Argument handling */
 int pyi_utils_initialize_args(const int argc, char *const argv[]);
+int pyi_utils_append_to_args(const char *arg);
 void pyi_utils_get_args(int *argc, char ***argv);
 void pyi_utils_free_args();
 
-/* Apple event handling */
-#if defined(__APPLE__) && defined(WINDOWED)
-/*
- * Watch for OpenDocument AppleEvents and add the files passed in to the
- * sys.argv command line on the Python side.
- *
- * This allows on Mac OS X to open files when a file is dragged and dropped
- * on the App icon in the OS X dock.
- */
-void pyi_process_apple_events(bool short_timeout);
-#endif  /* defined(__APPLE__) && defined(WINDOWED) */
-
 /* Magic pattern matching */
 extern const unsigned char MAGIC_BASE[8];
 uint64_t pyi_utils_find_magic_pattern(FILE *fp, const unsigned char *magic, size_t magic_len);
